<?php

namespace Drupal\entity_generic\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Implements basic entity class.
 */
abstract class Basic extends ContentEntityBase implements BasicInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

    $fields = parent::baseFieldDefinitions($entity_type);

    if ($entity_type->hasKey('created')) {
      $fields[$entity_type->getKey('created')] = BaseFieldDefinition::create('created')
        ->setLabel(t('Created'))
        ->setDisplayOptions('view', array(
          'label' => 'hidden',
          'type' => 'timestamp',
          'weight' => 10,
        ))
        ->setDisplayConfigurable('view', TRUE)
        ->setDisplayOptions('form', array(
          'type' => 'datetime_timestamp',
          'weight' => 10,
        ))
        ->setDisplayConfigurable('form', TRUE);
      if ($entity_type->isRevisionable()) {
        $fields[$entity_type->getKey('created')]->setRevisionable(TRUE);
      }
    }

    if ($entity_type->hasKey('changed')) {
      $fields['changed'] = BaseFieldDefinition::create('changed')
        ->setLabel(t('Changed'))
        ->setDisplayOptions('view', array(
          'label' => 'hidden',
          'type' => 'timestamp',
          'weight' => 20,
        ))
        ->setDisplayConfigurable('view', TRUE)
      ;
    }

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

}
