<?php

namespace Drupal\entity_generic\Entity;

use Drupal\Core\Entity\RevisionLogInterface;
use Drupal\user\EntityOwnerInterface;

/**
 * Defines an interface for simple entities.
 */
interface SimpleInterface extends BasicInterface, EntityOwnerInterface, RevisionLogInterface {

  /**
   * Denotes that the entity is disabled.
   */
  const ENTITY_DISABLED = 0;

  /**
   * Denotes that the entity is enabled.
   */
  const ENTITY_ENABLED = 1;

  /**
   * Returns the entity status.
   *
   * Unpublished entities are only visible to their authors and to administrators.
   *
   * @return bool
   *   TRUE if the entity is active.
   */
  public function isActive();

  /**
   * Gets the entity status.
   *
   * @return bool
   *   TRUE if the entity is active.
   */
  public function getStatus();

  /**
   * Sets the active status of an entity.
   *
   * @param bool $active
   *   TRUE to set this entity to active, FALSE to set it to inactive.
   *
   * @return \Drupal\entity_generic\Entity\SimpleInterface
   *   The called entity.
   */
  public function setStatus($active);

  /**
   * Gets the entity label.
   *
   * @return string
   *   Label of the entity.
   */
  public function getLabel();

  /**
   * Sets the entity label.
   *
   * @param string $label
   *   The entity label.
   *
   * @return \Drupal\entity_generic\Entity\SimpleInterface
   *   The called entity.
   */
  public function setLabel($label);

}
