<?php

namespace Drupal\entity_generic;

use Drupal\Core\Entity\Sql\SqlContentEntityStorage;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Defines the storage handler class for entities.
 *
 * This extends the base storage class, adding required special handling for
 * entities.
 */
class GenericStorage extends SqlContentEntityStorage implements GenericStorageInterface {

  /**
   * {@inheritdoc}
   */
  public function revisionIds(ContentEntityInterface $entity) {
    return $this->database->query(
      'SELECT vid FROM {generic_revision} WHERE id=:id ORDER BY vid',
      array(':id' => $entity->id())
    )->fetchCol();
  }

  /**
   * {@inheritdoc}
   */
  public function userRevisionIds(ContentEntityInterface $entity, AccountInterface $account) {
    return $this->database->query(
      'SELECT vid FROM {generic_data_revision} WHERE uid = :uid ORDER BY vid',
      array(':uid' => $account->id())
    )->fetchCol();
  }

  /**
   * {@inheritdoc}
   */
  public function countDefaultLanguageRevisions(ContentEntityInterface $entity) {
    return $this->database->query(
      'SELECT COUNT(*) FROM {generic_data_revision} WHERE id = :id AND default_langcode = 1',
      array(':id' => $entity->id())
    )->fetchField();
  }

  /**
   * {@inheritdoc}
   */
  public function updateType(ContentEntityInterface $entity, $old_type, $new_type) {
    return $this->database->update('entity_generic')
      ->fields(array('type' => $new_type))
      ->condition('type', $old_type)
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function clearRevisionsLanguage(ContentEntityInterface $entity, LanguageInterface $language) {
    return $this->database->update('entity_generic_revision')
      ->fields(array('langcode' => LanguageInterface::LANGCODE_NOT_SPECIFIED))
      ->condition('langcode', $language->getId())
      ->execute();
  }

}
